<?php
// $Id: aurigma.upload.inc,v 1.2.2.2 2009/03/29 23:13:33 awolfey Exp $

/**
 * @file
 * The aurigma uploader sends images to this file, which is a menu callback.
 *
 * Images are validated and saved, new nodes are pre-populated with default values.
 * Tokens are replaced if using token and filefield_paths.
 */

function aurigma_upload($type_name, $field_name, $nid) {
  if (!$_FILES || !$_POST) {
    return;
  }

  // Load cck include and get field instance.
  $field_instance = content_fields($field_name, $type_name);

  // We load the node we're editing and update any changed fields appearing above Aurigma
  // on the form. If  there's no $nid node_save will assign a real nid, but we need to assign uid.
  $node = node_load($nid);

  // If it's a new node, we set the defaults for this content type.
  if ($nid == 0) {
    global $user;
    $node->uid = $user->uid;
    $node->type = $type_name;

    $options = variable_get('node_options_'. $type_name, '');
    if (in_array('promote', $options)) {
      $node->promote = 1;
    }
    if (in_array('status', $options)) {
      $node->status = 1;
    }
    if (in_array('sticky', $options)) {
      $node->sticky = 1;
    }

    // We do a preliminary node_save() so we get a real node id.
    node_save($node);
  }
  $node->title = ($node->title == $_POST['title'] ? $node->title : $_POST['title']);
  $node->body = ($node->body == $_POST['body'] ? $node->body : $_POST['body']);
  $node->format = ($node->format == $_POST['format'] ? $node->format : $_POST['format']);

  // For new nodes where the user has not entered a title, we set one, because
  // it's always good to have a title.
  if (!$node->title) {
    form_set_error('title', $message = 'Please enter a title.', $reset = FALSE);
    $node->title = t('Temporary title - please replace.');

  }

  $fileCount = $_POST ["FileCount"];
  $dest = file_directory_temp();

  for ($i = 1; $i <= $fileCount; $i++) {
    $sourceFileField = "SourceFile_" . $i;
    $filename = $_FILES[$sourceFileField]['tmp_name'];
    $handle = fopen($filename, "r");
    $data = fread($handle, filesize($filename));
    fclose($handle);

    $filepath = $dest .'/'. $_FILES[$sourceFileField]['name'];

    if ($filepath = file_save_data($data, $filepath)) {

      // If we are using tokens, we replace the token and create the directory if necessary
      // Only checking for filefield_paths because it requires token.
      if (module_exists('filefield_paths')) {

        // We reuse some code from filefile paths module.
        $result = db_fetch_object(
          db_query("SELECT filename, filepath FROM {filefield_paths} WHERE type = '%s' AND field = '%s'", $node->type, $field_name)
        );
        $settings['filepath'] = unserialize($result->filepath);
        $filefield_path = $settings['filepath']['value'];

        // Add the tokens and other path info.
        $destination_path = $filefield_path;

        // Replace tokens with values.
        $destination_path = token_replace_multiple($destination_path, $types = array('node' => $node), $leading = '[', $trailing = ']', $options = array());
        aurigma_file_check_directories($destination_path, $mode = 1, $form_item = NULL);

        // Add back on the files directory before we save the file.
        $destination_path = file_directory_path() .'/'. $destination_path;

      }
      // If not using token we'll save to the widget's path or drupal's file directory.
      else if ($destination_path = filefield_widget_file_path($field_instance)){
      }

      else {
        $destination_path = file_directory_path();
      }

      // get the validators for this field
      $validators = filefield_widget_upload_validators($field_instance);

      // Save the new file.
      if (! $file = field_file_save_file($filepath, $validators, $destination_path)) {
        watchdog('Aurigma Upload', 'Could not save file', NULL, WATCHDOG_ERROR);
        file_delete($filepath);

        continue;
      }

      // Watchdog for successful file upload.
      watchdog('aurigma', t('%Image uploaded.', array('%Image' => $file['filename'])));
      file_delete($filepath);

      // Add the description or simply set the field.
      if (!$_POST[Description .'_'. $i]) {
        $file['data'] = array('description' => '');
      }
      else {
        $file['data'] = array('description' => $_POST[Description .'_'. $i]);
      }

      $node->{$field_name}[] = $file;

      // Build an array of new fids to use later.
      $newfids[] = $file['fid'];

    }

    // Something is wrong with the file. Message to user.
    else {
      drupal_set_message($message = 'The file '. $file->filename .' could not be saved.', $type = 'status', $repeat = TRUE);
    }
  }

  // save the node
  node_save($node);

  // Check for default alt and title, replace tokens if module exists then add
  // to the field's content table.
  if ($field_instance['widget']['alt']) {
    $altdata['alt'] =  $field_instance['widget']['alt'];
  }
  if ($field_instance['widget']['title']) {
    $altdata['title'] =  $field_instance['widget']['title'];
  }

  if (isset($altdata)) {
    if (module_exists('token')) {
      $altdata = token_replace($altdata, $types = 'global', $object = NULL, $leading = '[', $trailing = ']', $options = array());
    }

    $altdata = serialize($altdata);
    $aurigmatable = 'content_'. $field_name;
    $aurigmacolumn = $field_name .'_data';
    $aurigmafid = $field_name .'_fid';

    foreach ($newfids as $key => $fid) {
      $query = "UPDATE {%s} SET %s = '%s' WHERE %s = %d";
      db_query($query, $aurigmatable, $aurigmacolumn, $altdata, $aurigmafid, $fid);
    }
  }

  // Clear the cache so that the new records are loaded into the edit form.
  cache_clear_all('content:'. $node->nid .':'. $node->vid, 'cache_content');
  cache_clear_all('form_'. $_POST['form_build_id'], 'cache_form');

  exit;
}
